@@ -61,8 +61,6 @@ setReplaceMethod("rownames", "DataFrame",
                        stop("missing values not allowed in rownames")
                      if (length(value) != nrow(x))
                        stop("invalid rownames length")
-                     if (anyDuplicated(value))
-                       stop("duplicate rownames not allowed")
                      if (!is(value, "XStringSet"))
                        value <- as.character(value)
                    }
@@ -160,11 +158,14 @@ new_DataFrame <- function(listData=list(), nrows=NA, what="arguments")
         new2("DataFrame", nrows=nrows, listData=listData, check=FALSE)
 }
 
-DataFrame <- function(..., row.names = NULL, check.names = TRUE)
+DataFrame <- function(..., row.names = NULL, check.names = TRUE,
+                      stringsAsFactors)
 {
   ## build up listData, with names from arguments
   if (!isTRUEorFALSE(check.names))
     stop("'check.names' must be TRUE or FALSE")
+  if (!missing(stringsAsFactors))
+    warning("'stringsAsFactors' is ignored")
   nr <- 0
   listData <- list(...)
   varlist <- vector("list", length(listData))
@@ -174,21 +175,16 @@ DataFrame <- function(..., row.names = NULL, check.names = TRUE)
         metadata <- metadata(listData[[1L]])
     dotnames <- names(listData)
     if (is.null(dotnames)) {
-      emptynames <- rep.int(TRUE, length(listData))
-    } else {
-      emptynames <- !nzchar(dotnames)
+        dotnames <- rep("", length(listData))
     }
-    if (any(emptynames)) {
-      qargs <- as.list(substitute(list(...)))[-1L]
-      dotvalues <- sapply(qargs[emptynames], function(arg) deparse(arg)[1L])
-      names(listData)[emptynames] <- dotvalues
-    }
-    varnames <- as.list(names(listData))
+    qargs <- as.list(substitute(list(...)))[-1L]
+    varnames <- as.list(dotnames)
+    varnames[dotnames == ""] <- list(NULL)
     nrows <- ncols <- integer(length(varnames))
     for (i in seq_along(listData)) {
       element <- try(as(listData[[i]], "DataFrame"), silent = TRUE)
       if (inherits(element, "try-error"))
-        stop("cannot coerce class \"", class(listData[[i]]),
+        stop("cannot coerce class \"", class(listData[[i]])[1L],
              "\" to a DataFrame")
       nrows[i] <- nrow(element)
       ncols[i] <- ncol(element)
@@ -199,12 +195,15 @@ DataFrame <- function(..., row.names = NULL, check.names = TRUE)
         if ((length(dim(listData[[i]])) > 1L) || (ncol(element) > 1L) ||
              is.list(listData[[i]]))
           {
-            if (emptynames[i])
+            if (is.null(varnames[[i]]))
               varnames[[i]] <- colnames(element)
             else
               varnames[[i]] <- paste(varnames[[i]], colnames(element), sep = ".")
           }
       }
+      if (is.null(varnames[[i]])) {
+          varnames[[i]] <- deparse(qargs[[i]])[1L]
+      }
       if (missing(row.names))
         row.names <- rownames(element)
     }
@@ -233,8 +232,6 @@ DataFrame <- function(..., row.names = NULL, check.names = TRUE)
       stop("missing values in 'row.names'")
     if (length(varlist) && length(row.names) != nr)
       stop("invalid length of row names")
-    if (anyDuplicated(row.names))
-      stop("duplicate row names")
     row.names <- as.character(row.names)
   }
 
@@ -290,14 +287,10 @@ setMethod("extractROWS", "DataFrame",
     {
         i <- normalizeSingleBracketSubscript(i, x, exact=FALSE, allow.NAs=TRUE,
                                                    as.NSBS=TRUE)
-        slot(x, "listData", check=FALSE) <-
-            lapply(structure(seq_len(ncol(x)), names=names(x)),
-                   function(j) extractROWS(x[[j]], i))
+        slot(x, "listData", check=FALSE) <- lapply(as.list(x), extractROWS, i)
         slot(x, "nrows", check=FALSE) <- length(i)
-        if (!is.null(rownames(x))) {
-            slot(x, "rownames", check=FALSE) <-
-                make.unique(extractROWS(rownames(x), i))
-        }
+        if (!is.null(rownames(x)))
+            slot(x, "rownames", check=FALSE) <- extractROWS(rownames(x), i)
         x
     }
 )
@@ -326,7 +319,7 @@ setMethod("[", "DataFrame",
                 j <- normalizeSingleBracketSubscript(j, xstub)
             }
             new_listData <- extractROWS(x@listData, j)
-            new_mcols <- extractROWS(mcols(x), j)
+            new_mcols <- extractROWS(mcols(x, use.names=FALSE), j)
             x <- initialize(x, listData=new_listData,
                                elementMetadata=new_mcols)
             if (anyDuplicated(names(x)))
@@ -363,11 +356,10 @@ setMethod("[", "DataFrame",
         ans_rownames <- replaceROWS(ans_rownames, i, value_rownames)
     if (is.null(x_rownames))
         x_rownames <- as.character(seq_len(x_nrow))
-    ans_rownames <- replaceROWS(ans_rownames, seq_len(x_nrow), x_rownames)
-    make.unique(ans_rownames)
+    replaceROWS(ans_rownames, seq_len(x_nrow), x_rownames)
 }
 
-setMethod("replaceROWS", "DataFrame",
+setMethod("replaceROWS", c("DataFrame", "ANY"),
     function(x, i, value)
     {
         i <- normalizeSingleBracketSubscript(i, x, allow.append=TRUE,
@@ -566,8 +558,8 @@ setReplaceMethod("[", "DataFrame",
                    if (length(newcn)) {
                      oldcn <- head(colnames(x), length(x) - length(newcn))
                      colnames(x) <- make.unique(c(oldcn, newcn))
-                     if (!is.null(mcols(x)))
-                       mcols(x) <- replaceROWS(mcols(x),
+                     if (!is.null(mcols(x, use.names=FALSE)))
+                       mcols(x) <- replaceROWS(mcols(x, use.names=FALSE),
                                                tail(names(x), length(newcn)),
                                                DataFrame(NA))
                    }
@@ -576,7 +568,7 @@ setReplaceMethod("[", "DataFrame",
                      x@listData[notj] <-
                        lapply(x@listData[notj],
                               function(y) c(y, rep(NA, length(newrn))))
-                     x@rownames <- make.unique(c(rownames(x), newrn))
+                     x@rownames <- c(rownames(x), newrn)
                    }
                    x@nrows <- NROW(x[[1]]) # we should always have a column
                    x
@@ -592,11 +584,11 @@ hasS3Method <- function(f, signature) {
 
 droplevels.DataFrame <- function(x, except=NULL) {
   canDropLevels <- function(xi) {
-    hasNonDefaultMethod(droplevels, class(xi)) ||
+    hasNonDefaultMethod(droplevels, class(xi)[1L]) ||
       hasS3Method("droplevels", class(xi))
   }
   drop.levels <- vapply(x, canDropLevels, NA)
-  if (!is.null(except)) 
+  if (!is.null(except))
     drop.levels[except] <- FALSE
   x@listData[drop.levels] <- lapply(x@listData[drop.levels], droplevels)
   x
@@ -617,33 +609,61 @@ setAs("DataFrame", "data.frame",
         as.data.frame(from, optional=TRUE)
       })
 
-injectIntoScope <- function(x, ...) {
-  nms <- sapply(tail(substitute(list(...)), -1), deparse)
-  environment(x) <- list2env(setNames(list(...), nms), parent = environment(x))
-  x
-}
-
-.as.data.frame.DataFrame <- function(x, row.names=NULL, optional=FALSE, ...)
+.as.data.frame.DataFrame <- function(x, row.names=NULL, optional=FALSE,
+                                     stringsAsFactors=FALSE, ...)
 {
+    stopifnot(identical(stringsAsFactors, FALSE))
     if (length(list(...)))
         warning("Arguments in '...' ignored")
-    l <- as(x, "list")
-    if (is.null(row.names))
+    if (is.null(row.names)) {
         row.names <- rownames(x)
-    if (!length(l) && is.null(row.names))
-        row.names <- seq_len(nrow(x))
-    l <- lapply(l,
-                function(y) {
-                    if (is(y, "SimpleList") || is(y, "CompressedList"))
-                        y <- as.list(y)
-                    if (is.list(y))
-                        y <- I(y)
-                    y
-                })
-    IRanges.data.frame <- injectIntoScope(data.frame, as.data.frame)
-    do.call(IRanges.data.frame,
-            c(l, list(row.names=row.names),
-              check.names=!optional, stringsAsFactors=FALSE))
+        if (!is.null(row.names))
+            row.names <- make.unique(row.names)
+        else if (ncol(x) == 0L)
+            row.names <- seq_len(nrow(x))
+    }
+    old_option <- getOption("stringsAsFactors")
+    options(stringsAsFactors=FALSE)
+    on.exit(options(stringsAsFactors=old_option))
+    x_colnames <- colnames(x)
+    df_list <- lapply(setNames(seq_along(x), x_colnames),
+        function(j) {
+            col <- x[[j]]
+            if (is.data.frame(col))
+                return(col)
+            if (is(col, "DataFrame"))
+                return(as.data.frame(col, optional=optional))
+            ## If 'col is an AtomicList derivative (e.g. IntegerList,
+            ## CharacterList, etc...) or other List derivative that compares
+            ## recursively (i.e. not an IRanges, GRanges, or DNAStringSet,
+            ## etc... object), we turn it into an ordinary list. This is
+            ## because the "as.data.frame" method for List objects produces
+            ## this weird data frame:
+            ##   > as.data.frame(IntegerList(11:12, 21:23))
+            ##     group group_name value
+            ##   1     1       <NA>    11
+            ##   2     1       <NA>    12
+            ##   3     2       <NA>    21
+            ##   4     2       <NA>    22
+            ##   5     2       <NA>    23
+            ## which is not what we want here.
+            ## List derivatives that compare recursively should not need this
+            ## because they are expected to override the "as.data.frame" method
+            ## for List objects with a method that returns a data.frame with
+            ## one row per list element.
+            if (is(col, "List") && pcompareRecursively(col))
+                col <- as.list(col)
+            if (is.list(col))
+                col <- I(col)
+            df <- as.data.frame(col, optional=optional)
+            if (is.null(colnames(col)) && ncol(df) == 1L)
+                colnames(df) <- x_colnames[[j]]
+            df
+        })
+    do.call(data.frame,
+            c(df_list, list(row.names=row.names,
+                            check.names=!optional,
+                            stringsAsFactors=FALSE)))
 }
 setMethod("as.data.frame", "DataFrame", .as.data.frame.DataFrame)
 
@@ -703,9 +723,8 @@ setAs("xtabs", "DataFrame",
       names(df) <- character()
     as(df, "DataFrame")
   } else {
-    row.names <- if (!anyDuplicated(names(from))) names(from) else NULL
     ans <- new_DataFrame(setNames(list(from), "X"), nrows=length(from))
-    ans@rownames <- row.names
+    ans@rownames <- names(from)
     ans
   }
 }
@@ -714,9 +733,9 @@ setAs("ANY", "DataFrame", .defaultAsDataFrame)
 
 .VectorAsDataFrame <- function(from) {
   ans <- .defaultAsDataFrame(from)
-  if (!is.null(mcols(from))) {
-    ans <- cbind(ans, mcols(from))
-  }
+  from_mcols <- mcols(from, use.names=FALSE)
+  if (!is.null(from_mcols))
+    ans <- cbind(ans, from_mcols)
   ans
 }
 
@@ -728,18 +747,11 @@ setAs("Vector", "DataFrame", .VectorAsDataFrame)
 ## is this a bug or a feature?
 setAs("list", "DataFrame",
       function(from) {
-        do.call(DataFrame, c(from, check.names = is.null(names(from))))
+        do.call(DataFrame, c(from, list(check.names=is.null(names(from)))))
       })
 
 setAs("NULL", "DataFrame", function(from) as(list(), "DataFrame"))
 
-### FIXME: only exists due to annoying S4 warning due to its caching of
-### coerce methods.
-setAs("integer", "DataFrame",
-      function(from) {
-        selectMethod("coerce", c("vector", "DataFrame"))(from)
-      })
-
 setAs("AsIs", "DataFrame",
       function(from) {
         new_DataFrame(setNames(list(drop_AsIs(from)), "X"))
@@ -747,14 +759,36 @@ setAs("AsIs", "DataFrame",
 
 setAs("ANY", "DataTable_OR_NULL", function(from) as(from, "DataFrame"))
 
-### Turn an ordinary list into a DataFrame in the most possibly
-### straightforward way.
 setMethod("coerce2", "DataFrame",
     function(from, to)
     {
-        if (class(from) != "list")
-            return(callNextMethod())
-        new_DataFrame(from, what="list elements")
+        if (class(from) == "list") {
+            ## Turn an ordinary list into a DataFrame in the most possibly
+            ## straightforward way.
+            return(new_DataFrame(from, what="list elements"))
+        }
+        ## Some objects like SplitDataFrameList have a "dim" method that
+        ## returns a non-MULL object (a matrix!) even though they don't have
+        ## an array-like (or matrix-like) semantic.
+        from_dim <- dim(from)
+        if (length(from_dim) == 2L && !is.matrix(from_dim)) {
+            to_class <- class(to)
+            if (is(from, to_class))
+                return(from)
+            ans <- as(from, to_class, strict=FALSE)
+            if (!identical(dim(ans), from_dim))
+                stop(wmsg("coercion of ", class(from), " object to ", to_class,
+                          " didn't preserve its dimensions"))
+            ## Try to restore the dimnames if they were lost or altered.
+            from_dimnames <- dimnames(from)
+            if (!identical(dimnames(ans), from_dimnames)) {
+                tmp <- try(`dimnames<-`(ans, value=from_dimnames), silent=TRUE)
+                if (!inherits(tmp, "try-error"))
+                    ans <- tmp
+            }
+            return(ans)
+        }
+        callNextMethod()
     }
 )
 
@@ -762,94 +796,182 @@ setMethod("coerce2", "DataFrame",
 ### Combining.
 ###
 
-cbind.DataFrame <- function(..., deparse.level = 1) {
-  DataFrame(..., check.names=FALSE)
+### Return an integer matrix with 1 column per object in 'objects' and 1 row
+### per column in 'x'.
+.make_colmaps <- function(x, objects)
+{
+    x_colnames <- colnames(x)
+    x_ncol <- length(x_colnames)
+    map_x_colnames_to_object_colnames <- function(object_colnames) {
+        if (length(object_colnames) != x_ncol)
+            stop(wmsg("the DataFrame objects to rbind ",
+                      "must have the same number of columns"))
+        colmap <- selectHits(findMatches(x_colnames, object_colnames),
+                             select="first", nodup=TRUE)
+        if (anyNA(colmap))
+            stop(wmsg("the DataFrame objects to rbind ",
+                      "must have the same colnames"))
+        colmap
+    }
+    if (length(objects) == 0L) {
+        colmaps <- integer(0)
+    } else {
+        colmaps <- lapply(objects,
+            function(object)
+                map_x_colnames_to_object_colnames(colnames(object)))
+        colmaps <- unlist(colmaps, use.names=FALSE)
+    }
+    dim(colmaps) <- c(x_ncol, length(objects))
+    colmaps
+}
+
+### A thin wrapper around bindROWS()
+###
+### If all the columns to bind have the same type, the result of the binding
+### should be a column of that type (endomorphism). Note that this is what
+### happens with ordinary data frames. So we could just use bindROWS() for
+### that. However, when the columns to bind have mixed types, bindROWS()
+### won't necessarily do the right thing.
+### The purpose of the wrapper below is to improve handling of mixed type
+### columns by pre-processing some of them before calling bindROWS() on them.
+### More precisely, it tries to work around the 2 following problems that
+### direct use of bindROWS() on mixed type columns would pose:
+###  1) When the columns to bind are a mix of Rle and non-Rle objects,
+###     the type of the column returned by bindROWS() depends on the type
+###     of cols[[1]]. More precisely it's an Rle if and only if cols[[1]]
+###     is an Rle. The wrapper below **mitigate** this by decoding the Rle
+###     columns first. Note that this is a mitigation process only. For
+###     example it will help if Rle columns are mixed with atomic vectors
+###     or factors, but it won't help if the cols[[1]] is an Rle and the
+###     other columns are IntegerList objects.
+###  2) When the columns to bind are a mix of atomic vectors and factors,
+###     bindROWS() would **always** return an atomic vector (whatever
+###     cols[[1]] is, i.e. atomic vector or factor). However we **always**
+###     want a factor. This is an intended deviation with respect to what
+###     rbind() does on ordinary data frames where the 1st data frame
+###     involved in the binding operation governs (i.e. a column in the
+###     result will be atomic vector or factor depending on what the
+###     corresponding column in the 1st data frame is).
+.bind_cols_along_their_ROWS <- function(cols)
+{
+    is_Rle <- vapply(cols, is, logical(1L), "Rle")
+    if (any(is_Rle) && !all(is_Rle))
+        cols[is_Rle] <- lapply(cols[is_Rle], decodeRle)
+    is_factor <- vapply(cols, is.factor, logical(1L))
+    if (any(is_factor)) {
+        cols <- lapply(cols, as.factor)
+        all_levels <- unique(unlist(lapply(cols, levels), use.names=FALSE))
+        cols <- lapply(cols, factor, all_levels)
+    }
+    bindROWS(cols[[1L]], cols[-1L])
+}
+
+### Argument 'ignore.mcols' is ignored.
+.rbind_DataFrame_objects <-
+    function(x, objects=list(), use.names=TRUE, ignore.mcols=FALSE, check=TRUE)
+{
+    if (!isTRUEorFALSE(use.names))
+        stop("'use.names' must be TRUE or FALSE")
+    objects <- prepare_objects_to_bind(x, objects)
+    all_objects <- c(list(x), objects)
+    has_rows <- vapply(all_objects, nrow, integer(1L), USE.NAMES=FALSE) > 0L
+    has_cols <- vapply(all_objects, ncol, integer(1L), USE.NAMES=FALSE) > 0L
+    if (!any(has_rows)) {
+        if (!any(has_cols))
+            return(x)
+        return(all_objects[[which(has_cols)[[1L]]]])
+    }
+    all_objects <- all_objects[has_rows]
+
+    ## From now on, all the objects to bind have rows.
+    x <- all_objects[[1L]]
+    objects <- all_objects[-1L]
+    colmaps <- .make_colmaps(x, objects)
+    if (ncol(x) == 0L) {
+        ans_listData <- x@listData
+        ans_nrow <- sum(unlist(lapply(all_objects, nrow), use.names=FALSE))
+    } else {
+        ans_listData <- lapply(setNames(seq_along(x), colnames(x)),
+            function(i) {
+                 x_col <- x[[i]]
+                 other_cols <- lapply(seq_along(objects),
+                                      function(j) objects[[j]][[colmaps[i, j]]])
+                 .bind_cols_along_their_ROWS(c(list(x_col), other_cols))
+            }
+        )
+        ans_nrow <- NROW(ans_listData[[1L]])
+    }
+    if (use.names) {
+        ## Bind the rownames.
+        ans_rownames <- unlist(lapply(all_objects, rownames), use.names=FALSE)
+        if (!is.null(ans_rownames)) {
+            if (length(ans_rownames) != ans_nrow) {
+                ## What we do here is surprising and inconsistent with
+                ## ordinary data frames.
+                ## TODO: Maybe reconsider this?
+                ans_rownames <- NULL  # why?
+            }
+        }
+    } else {
+        ans_rownames <- NULL
+    }
+    BiocGenerics:::replaceSlots(x, listData=ans_listData,
+                                   nrows=ans_nrow,
+                                   rownames=ans_rownames,
+                                   check=check)
+}
+
+setMethod("bindROWS", "DataFrame", .rbind_DataFrame_objects)
+
+### The "rbind" method for DataFrame objects is just a thin wrapper around
+### a call to bindROWS().
+### Argument 'deparse.level' is ignored.
+rbind.DataFrame <- function(..., deparse.level=1)
+{
+    objects <- list(...)
+    bindROWS(objects[[1L]], objects[-1L], check=FALSE)
+}
+
+setMethod("rbind", "DataFrame", rbind.DataFrame)
+
+### Argument 'deparse.level' is ignored.
+cbind.DataFrame <- function(..., deparse.level=1)
+{
+    ## It's important that the call to DataFrame() below is able to deparse
+    ## the arguments in ... so for example
+    ##   b <- 11:13
+    ##   selectMethod("cbind", "DataFrame")(b)
+    ## returns a DataFrame with a column named "b".
+    ## This prevents us from calling DataFrame() via do.call() e.g. we can't
+    ## do something like
+    ##   objects <- delete_NULLs(list(...))
+    ##   do.call(DataFrame, c(objects, list(check.names=FALSE)))
+    ## because then DataFrame() wouldn't be able to deparse what was in ...
+    ## and selectMethod("cbind", "DataFrame")(b) would produce a DataFrame
+    ## with a column named "11:13".
+    DataFrame(..., check.names=FALSE)
 }
 
 setMethod("cbind", "DataFrame", cbind.DataFrame)
 
-rbind.DataFrame <- function(..., deparse.level = 1) {
-  do.call(rbind, lapply(list(...), as, "DataFrame"))
-}
-
-setMethod("rbind", "DataFrame", function(..., deparse.level=1) {
-  args <- list(...)    
-  
-  hasrows <- elementNROWS(args) > 0L
-  hascols <- vapply(args, ncol, integer(1L)) > 0L
-
-  ans <- NULL
-  if (!any(hasrows | hascols)) {
-    ans <- DataFrame()
-  } else if (!any(hasrows)) {
-    ans <- args[[which(hascols)[1L]]]
-  } else if (sum(hasrows) == 1) {
-    ans <- args[[which(hasrows)]]
-  } else {
-    args <- args[hasrows]
-  }
-
-  df <- args[[1L]]
-
-  ans_metadata <- metadata(df)
-  ans_mcols <- mcols(df)
-  
-  if (!is.null(ans)) {
-      metadata(ans) <- ans_metadata
-      mcols(ans) <- ans_mcols
-      return(ans)
-  }
-  
-  for (i in 2:length(args)) {
-    if (ncol(df) != ncol(args[[i]]))
-      stop("number of columns for arg ", i, " do not match those of first arg")
-    if (!identical(colnames(df), colnames(args[[i]])))
-      stop("column names for arg ", i, " do not match those of first arg")
-  }
-
-  if (ncol(df) == 0) {
-    ans <- DataFrame()
-    ans@nrows <- sum(unlist(lapply(args, nrow), use.names=FALSE))
-  } else {
-    cols <- lapply(colnames(df), function(cn) {
-      cols <- lapply(args, `[[`, cn)
-      isRle <- vapply(cols, is, logical(1L), "Rle")
-      if (any(isRle) && !all(isRle)) { # would fail dispatch to c,Rle
-        cols[isRle] <- lapply(cols[isRle], decodeRle)
-      }
-      isFactor <- vapply(cols, is.factor, logical(1L))
-      if (any(isFactor)) {
-        cols <- lapply(cols, as.factor)
-        levs <- unique(unlist(lapply(cols, levels), use.names=FALSE))
-        cols <- lapply(cols, factor, levs)
-      }
-      matrix_like <- length(dim(cols[[1]])) == 2L
-      if (matrix_like) {
-        combined <- do.call(rbind, unname(cols))
-        ### FIXME: handle the general array case (arbind)
-      } else {
-        combined <- do.call(c, unname(cols))
-      }
-      if (any(isFactor))
-        combined <- structure(combined, class="factor", levels=levs)
-      combined
-    })
-    names(cols) <- colnames(df)
-    ans <- new_DataFrame(cols, nrows=NROW(cols[[1]]))
-  }
-
-  rn <- unlist(lapply(args, rownames), use.names=FALSE)
-  if (!is.null(rn)) {
-    if (length(rn) != nrow(ans)) {
-      rn <- NULL
-    } else if (anyDuplicated(rn))
-      rn <- make.unique(rn, sep = "")
-  }
-  rownames(ans) <- rn
-  
-  metadata(ans) <- ans_metadata
-  mcols(ans) <- ans_mcols
- 
-  ans
-})
+### If we didn't define this method, calling c() on DataFrame objects would
+### call the "c" method for Vector objects, which just delegates to bindROWS()
+### so the binding would happen along the rows. This is not what we want so we
+### overwrite the "c" method for Vector objects with a method that binds along
+### the columns.
+setMethod("c", "DataFrame",
+    function(x, ..., ignore.mcols=FALSE, recursive=FALSE)
+    {
+        if (!isTRUEorFALSE(ignore.mcols))
+            stop("'ignore.mcols' must be TRUE or FALSE")
+        if (!identical(recursive, FALSE))
+            stop(wmsg("\"c\" method for DataFrame objects ",
+                      "does not support the 'recursive' argument"))
+        objects <- unname(delete_NULLs(list(x, ...)))
+        ans <- do.call(cbind, objects)
+        if (ignore.mcols)
+            mcols(ans) <- NULL
+        ans
+    }
+)
 
