@@ -266,7 +266,7 @@ setMethod("updateObject", "Hits",
 {
     new_Hits("SortedByQueryHits", from(from), to(from),
                                   nLnode(from), nRnode(from),
-                                  mcols(from))
+                                  mcols(from, use.names=FALSE))
 }
 setAs("Hits", "SortedByQueryHits", .from_Hits_to_SortedByQueryHits)
 
@@ -309,13 +309,14 @@ setMethod("extractROWS", "SortedByQueryHits",
 ### Displaying
 ###
 
-setMethod("classNameForDisplay", "Hits", function(x) "Hits")
-setMethod("classNameForDisplay", "SelfHits", function(x) "SelfHits")
+setMethod("classNameForDisplay", "SortedByQueryHits",
+    function(x) sub("^SortedByQuery", "", class(x))
+)
 
 .make_naked_matrix_from_Hits <- function(x)
 {
     x_len <- length(x)
-    x_mcols <- mcols(x)
+    x_mcols <- mcols(x, use.names=FALSE)
     x_nmc <- if (is.null(x_mcols)) 0L else ncol(x_mcols)
     ans <- cbind(from=as.character(from(x)),
                  to=as.character(to(x)))
@@ -334,7 +335,7 @@ showHits <- function(x, margin="", print.classinfo=FALSE,
 {
     x_class <- class(x)
     x_len <- length(x)
-    x_mcols <- mcols(x)
+    x_mcols <- mcols(x, use.names=FALSE)
     x_nmc <- if (is.null(x_mcols)) 0L else ncol(x_mcols)
     cat(classNameForDisplay(x), " object with ",
         x_len, " hit", ifelse(x_len == 1L, "", "s"),
@@ -402,12 +403,12 @@ setMethod("show", "Hits",
 .concatenate_Hits_objects <-
     function(x, objects=list(), use.names=TRUE, ignore.mcols=FALSE, check=TRUE)
 {
-    objects <- prepare_objects_to_concatenate(x, objects)
+    objects <- prepare_objects_to_bind(x, objects)
     .check_that_Hits_objects_are_concatenable(x, objects)
     callNextMethod()
 }
 
-setMethod("concatenateObjects", "Hits", .concatenate_Hits_objects)
+setMethod("bindROWS", "Hits", .concatenate_Hits_objects)
 
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -423,34 +424,75 @@ setMethod("sort", "SortedByQueryHits",
     callNextMethod()
 })
 
+
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### selectHits()
 ###
 
-selectHits <- function(x,
-                       select=c("all", "first", "last", "arbitrary", "count"),
-                       rank)
+### Return an integer vector parallel to the query (i.e. of length
+### 'nLnode(hits)') except when select="all", in which case it's a no-op.
+###
+### 'nodup' must be TRUE or FALSE (the default) and can only be set to TRUE
+### when 'select' is "first", "last" or "arbitrary", and when the input hits
+### are sorted by query. When 'nodup=TRUE', a given element in the subject is
+### not allowed to be assigned to more than one element in the query, which is
+### achieved by following a simple "first come first served" pairing strategy.
+### So the returned vector is guaranteed to contain unique non-NA values.
+### Note that such vector represents a mapping between the query and subject
+### that is one-to-zero-or-one in *both* directions. So it represents a
+### pairing between the elements in query and subject, where a given element
+### belongs to at most one pair.
+### A note about the "first come first served" pairing strategy: This strategy
+### is simple and fast, but, in general, it won't achieve a "maximal pairing"
+### (i.e. a pairing with the most possible number of pairs) for a given input
+### Hits object. However it actually does produce a maximal pairing if the
+### Hits object is the result of call to findMatches() (with select="all")'.
+### Also, in that case, this pairing strategy is symetric i.e. the resulting
+### pairs are not affected by switching 'x' and 'table' in the call to
+### findMatches() (or by transposing the input Hits object).
+###
+### Finally note that when 'select' is "first" or "last" and 'nodup' is FALSE,
+### or when 'select' is "count", the output of selectHits() is not affected
+### by the order of the hits in the input Hits object.
+selectHits <- function(hits,
+    select=c("all", "first", "last", "arbitrary", "count"),
+    nodup=FALSE,
+    rank)
 {
-    if (!is(x, "Hits"))
-        stop("'x' must be a Hits object")
+    if (!is(hits, "Hits"))
+        stop("'hits' must be a Hits object")
     select <- match.arg(select)
+    if (!isTRUEorFALSE(nodup))
+        stop(wmsg("'nodup' must be TRUE or FALSE"))
+    if (nodup && !(select %in% c("first", "last", "arbitrary")))
+        stop(wmsg("'nodup=TRUE' is only supported when 'select' ",
+                  "is \"first\", \"last\", or \"arbitrary\""))
+    if (!missing(rank) && (!(select %in% c("first", "last")) || nodup))
+        stop(wmsg("'rank' is only supported when 'select' ",
+                  "is \"first\" or \"last\" and 'nodup' is FALSE"))
     if (select == "all")
-        return(x)
-    to <- to(x)
+        return(hits)  # no-op
+
+    hits_from <- from(hits)
+    hits_to <- to(hits)
+    hits_nLnode <- nLnode(hits)
+    hits_nRnode <- nRnode(hits)
+
     if (!missing(rank)) {
-        r <- rank(x, ties.method="first", by=rank)
+        r <- rank(hits, ties.method="first", by=rank)
         revmap <- integer()
-        revmap[r] <- to
-        to <- r
+        revmap[r] <- hits_to
+        hits_to <- r
     }
-    ans <- .Call2("select_hits", from(x), to, nLnode(x), select,
-                  PACKAGE="S4Vectors")
-    if (!missing(rank)) {
+    ans <- .Call2("select_hits", hits_from, hits_to, hits_nLnode, hits_nRnode,
+                                 select, nodup,
+                                 PACKAGE="S4Vectors")
+    if (!missing(rank))
         ans <- revmap[ans]
-    }
     ans
 }
 
+
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### breakTies()
 ###
@@ -463,11 +505,12 @@ breakTies <- function(x, method=c("first", "last"), rank) {
     if (!is(x, "Hits"))
         stop("'x' must be a Hits object")
     method <- match.arg(method)
-    to <- selectHits(x, method, rank)
+    to <- selectHits(x, method, rank=rank)
     .new_Hits("SortedByQueryHits", which(!is.na(to)), to[!is.na(to)],
               nLnode(x), nRnode(x), NULL)
 }
 
+
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### revmap()
 ###
@@ -484,7 +527,8 @@ breakTies <- function(x, method=c("first", "last"), rank) {
 ###     'revmap_Hits(revmap_Hits(x))' brings back 'x' but with the hits in it
 ###     now "fully sorted".
 revmap_Hits <- function(x)
-    new_Hits(class(x), to(x), from(x), nRnode(x), nLnode(x), mcols(x))
+    new_Hits(class(x), to(x), from(x), nRnode(x), nLnode(x),
+                       mcols(x, use.names=FALSE))
 
 ### FIXME: Replace this with "revmap" method for Hits objects.
 t.Hits <- function(x) t(x)
@@ -573,7 +617,7 @@ remapHits <- function(x, Lnodes.remapping=NULL, new.nLnode=NA,
                       "are < 1, or > 'new.nRnode'"))
         x_to <- Rnodes.remapping[x_to]
     }
-    x_mcols <- mcols(x)
+    x_mcols <- mcols(x, use.names=FALSE)
     add_counts <- function(counts) {
         if (is.null(x_mcols))
             return(DataFrame(counts=counts))
