@@ -242,7 +242,7 @@ setMethod("extractROWS", c("Rle", "RangeNSBS"),
         range_start <- range[[1L]]
         range_end <- range[[2L]]
         ans <- extract_range_from_Rle(x, range_start, range_end)
-        mcols(ans) <- extractROWS(mcols(x), i)
+        mcols(ans) <- extractROWS(mcols(x, use.names=FALSE), i)
         ans
     }
 )
@@ -251,7 +251,7 @@ setMethod("extractROWS", c("Rle", "NSBS"),
     function(x, i)
     {
         ans <- extract_positions_from_Rle(x, as.integer(i))
-        mcols(ans) <- extractROWS(mcols(x), i)
+        mcols(ans) <- extractROWS(mcols(x, use.names=FALSE), i)
         ans
     }
 )
@@ -272,7 +272,7 @@ setMethod("[", "Rle",
 ### The replaced elements in 'x' must get their metadata columns from 'value'.
 ### See this thread on bioc-devel:
 ###   https://stat.ethz.ch/pipermail/bioc-devel/2015-November/008319.html
-setMethod("replaceROWS", "Rle",
+setMethod("replaceROWS", c("Rle", "ANY"),
     function(x, i, value)
     {
         ## FIXME: Right now, the subscript 'i' is turned into an IRanges
@@ -288,7 +288,8 @@ setMethod("replaceROWS", "Rle",
         lv <- length(value)
         if (lv != 1L) {
             ans <- Rle(replaceROWS(decodeRle(x), i, as.vector(value)))
-            mcols(ans) <- replaceROWS(mcols(x), i, mcols(value))
+            mcols(ans) <- replaceROWS(mcols(x, use.names=FALSE), i,
+                                      mcols(value, use.names=FALSE))
             return(ans)
         }
 
@@ -372,7 +373,8 @@ setMethod("replaceROWS", "Rle",
         if (isFactorRle)
             values <- dummy_value[values]
         ans <- Rle(values, unlist(lapply(subseqs, "[[", "lengths")))
-        mcols(ans) <- replaceROWS(mcols(x), i, mcols(value))
+        mcols(ans) <- replaceROWS(mcols(x, use.names=FALSE), i,
+                                  mcols(value, use.names=FALSE))
         ans
     }
 )
@@ -592,7 +594,7 @@ setMethod("rep", "Rle",
 .concatenate_Rle_objects <-
     function(x, objects=list(), use.names=TRUE, ignore.mcols=FALSE, check=TRUE)
 {
-    objects <- prepare_objects_to_concatenate(x, objects)
+    objects <- prepare_objects_to_bind(x, objects)
     all_objects <- c(list(x), objects)
 
     ## 1. Take care of the parallel slots
@@ -622,7 +624,7 @@ setMethod("rep", "Rle",
                                      check=check)
 }
 
-setMethod("concatenateObjects", "Rle", .concatenate_Rle_objects)
+setMethod("bindROWS", "Rle", .concatenate_Rle_objects)
 
 setMethod("append", c("Rle", "vector"),
           function (x, values, after = length(x)) {
