@@ -78,12 +78,12 @@ setMethod("ROWNAMES", "Vector", function(x) names(x))
 ### 3 accessors for the same slot: elementMetadata(), mcols(), and values().
 ### mcols() is the recommended one, use of elementMetadata() or values() is
 ### discouraged.
-setGeneric("elementMetadata",
-    function(x, use.names=FALSE, ...) standardGeneric("elementMetadata")
+setGeneric("elementMetadata", signature="x",
+    function(x, use.names=TRUE, ...) standardGeneric("elementMetadata")
 )
 
 setMethod("elementMetadata", "Vector",
-    function(x, use.names=FALSE, ...)
+    function(x, use.names=TRUE, ...)
     {
         if (!isTRUEorFALSE(use.names))
             stop("'use.names' must be TRUE or FALSE")
@@ -94,12 +94,12 @@ setMethod("elementMetadata", "Vector",
     }
 )
 
-setGeneric("mcols",
-    function(x, use.names=FALSE, ...) standardGeneric("mcols")
+setGeneric("mcols", signature="x",
+    function(x, use.names=TRUE, ...) standardGeneric("mcols")
 )
 
 setMethod("mcols", "Vector",
-    function(x, use.names=FALSE, ...)
+    function(x, use.names=TRUE, ...)
         elementMetadata(x, use.names=use.names, ...)
 )
 
@@ -150,7 +150,7 @@ setMethod("is.na", "Vector", function(x) rep(FALSE, length(x)))
             msg <- c(msg, paste0("'", what, "' is not parallel to 'x'"))
         }
     }
-    tmp <- mcols(x)
+    tmp <- mcols(x, use.names=FALSE)
     if (!(is.null(tmp) || nrow(tmp) == x_len)) {
         msg <- c(msg, "'mcols(x)' is not parallel to 'x'")
     }
@@ -174,11 +174,11 @@ setMethod("is.na", "Vector", function(x) rep(FALSE, length(x)))
 
 .valid.Vector.mcols <- function(x)
 {
-    x_mcols <- mcols(x)
-    if (!is(x_mcols, "DataTable_OR_NULL"))
-        return("'mcols(x)' must be a DataTable object or NULL")
+    x_mcols <- mcols(x, use.names=FALSE)
     if (is.null(x_mcols))
         return(NULL)
+    if (!is(x_mcols, "DataTable"))
+        return("'mcols(x)' must be a DataTable object or NULL")
     ## 'x_mcols' is a DataTable object.
     x_mcols_rownames <- rownames(x_mcols)
     if (is.null(x_mcols_rownames))
@@ -304,7 +304,8 @@ makeFixedColumnEnv <- function(x, parent, tform = identity) {
 }
 
 setMethod("as.env", "Vector", function(x, enclos, tform = identity) {
-  addSelfRef(x, makeFixedColumnEnv(x, as.env(mcols(x), enclos, tform), tform))
+  parent <- as.env(mcols(x, use.names=FALSE), enclos, tform)
+  addSelfRef(x, makeFixedColumnEnv(x, parent, tform))
 })
 
 as.list.Vector <- function(x, ...) as.list(x, ...)
@@ -408,7 +409,7 @@ setMethod("[", "Vector",
     {
         ans <- subset_along_ROWS(x, i, , ..., drop=drop)
         if (!missing(j))
-            mcols(ans) <- mcols(ans)[ , j, drop=FALSE]
+            mcols(ans) <- mcols(ans, use.names=FALSE)[ , j, drop=FALSE]
         ans
     }
 )
@@ -470,43 +471,43 @@ setReplaceMethod("[", "Vector",
     }
 )
 
-### Work on any Vector object on which c() and extractROWS() work.
-### Assume 'value' is compatible with 'x'.
-setMethod("replaceROWS", "Vector",
+### Work on any Vector object on which bindROWS() and extractROWS() work.
+### Assume that 'value' is compatible with 'x'.
+setMethod("replaceROWS", c("Vector", "ANY"),
     function(x, i, value)
     {
         i <- normalizeSingleBracketSubscript(i, x, as.NSBS=TRUE)
+        stopifnot(length(i) == NROW(value))
 
-        ## --<1>-- Concatenate 'x' and 'value' with c() -----
+        ## --<1>-- Concatenate 'x' and 'value' with bindROWS() -----
 
-        ## We assume that concatenateObjects() works on objects of
-        ## class 'class(x)' and does the right thing i.e. that it
-        ## returns an object of the same class as 'x' and of length
-        ## 'length(x) + length(value)'. We skip validation.
-        ans <- concatenateObjects(x, list(value), check=FALSE)
+        ## We assume that bindROWS() works on objects of class 'class(x)'
+        ## and does the right thing i.e. that it returns an object of the
+        ## same class as 'x' and of NROW 'NROW(x) + NROW(value)'. We skip
+        ## validation.
+        ans <- bindROWS(x, list(value), check=FALSE)
 
-        ## --<2>-- Subset 'c(x, value)' with extractROWS() -----
+        ## --<2>-- Subset 'ans' with extractROWS() -----
 
-        idx <- replaceROWS(seq_along(x), i, seq_along(value) + length(x))
+        idx <- replaceROWS(seq_along(x), i, seq_along(value) + NROW(x))
         ## Because of how we constructed it, 'idx' is guaranteed to be a valid
         ## subscript to use in 'extractROWS(ans, idx)'. By wrapping it inside a
         ## NativeNSBS object, extractROWS() won't waste time checking it or
         ## trying to normalize it.
-        idx <- NativeNSBS(idx, length(ans), TRUE, FALSE)
+        idx <- NativeNSBS(idx, NROW(ans), TRUE, FALSE)
         ## We assume that extractROWS() works on an object of class 'class(x)'.
         ## For some objects (e.g. Hits), extractROWS() will take care of
         ## validating the returned object.
         ans <- extractROWS(ans, idx)
 
-        ## --<3>-- Restore the original decoration -----
+        ## --<3>-- Restore the original names -----
 
-        metadata(ans) <- metadata(x)
         names(ans) <- names(x)
-        ## However, we want the replaced elements in 'x' to get their
-        ## metadata columns from 'value' so we do not restore the original
-        ## metadata columns. See this thread on bioc-devel:
+        ## Note that we want the elements coming from 'value' to bring their
+        ## metadata columns into 'x' so we do NOT restore the original metadata
+        ## columns. See this thread on bioc-devel:
         ##  https://stat.ethz.ch/pipermail/bioc-devel/2015-November/008319.html
-        #mcols(ans) <- mcols(x)
+        #mcols(ans) <- mcols(x, use.names=FALSE)
 
         ans
     }
@@ -522,9 +523,10 @@ subset.Vector <- function(x, ...) subset(x, ...)
 subset_Vector <- function(x, subset, select, drop=FALSE, ...)
 {
     i <- evalqForSubset(subset, x, ...)
-    if (!is.null(mcols(x))) {
-        j <- evalqForSelect(select, mcols(x), ...)
-        mcols(x) <- mcols(x)[ , j, drop=FALSE]
+    x_mcols <- mcols(x, use.names=FALSE)
+    if (!is.null(x_mcols)) {
+        j <- evalqForSelect(select, x_mcols, ...)
+        mcols(x) <- x_mcols[ , j, drop=FALSE]
     }
     x[i, drop=drop]
 }
@@ -552,9 +554,18 @@ setMethod("rep.int", "Vector", rep.int_along_ROWS)
 ## NOT exported.
 revROWS <- function(x) extractROWS(x, rev(seq_len(NROW(x))))
 
+### S3/S4 combo for rev.Vector
+rev.Vector <- revROWS
+setMethod("rev", "Vector", revROWS)
+
+## NOT exported.
+repROWS <- function(x, ...) extractROWS(x, rep(seq_len(NROW(x)), ...))
+
+setMethod("rep", "Vector", repROWS)
+
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### Concatenation
+### Concatenation along the ROWS
 ###
 ### Note that supporting "extractROWS" and "c" makes "replaceROWS" (and thus
 ### "[<-") work out-of-the-box!
@@ -562,53 +573,52 @@ revROWS <- function(x) extractROWS(x, rev(seq_len(NROW(x))))
 
 ### Somewhat painful that we do not always have a DataFrame in elementMetadata
 ensureMcols <- function(x) {
-  mc <- mcols(x)
-  if (is.null(mc)) {
-    mc <- make_zero_col_DataFrame(length(x))
-  }
-  mc
+  ans <- mcols(x, use.names=FALSE)
+  if (is.null(ans))
+    ans <- make_zero_col_DataFrame(length(x))
+  ans
 }
 
-rbind_mcols <- function(x, ...)
+rbind_mcols <- function(...)
 {
-    args <- c(if (!missing(x)) list(x), list(...))
-    mcols_list <- lapply(args, mcols)
+    objects <- unname(list(...))
+    mcols_list <- lapply(objects, mcols, use.names=FALSE)
     if (length(mcols_list) == 1L)
         return(mcols_list[[1L]])
     mcols_is_null <- sapply_isNULL(mcols_list)
     if (all(mcols_is_null))
         return(NULL)
     mcols_list[mcols_is_null] <- lapply(
-        args[mcols_is_null],
-        function(arg) make_zero_col_DataFrame(length(arg))
+        objects[mcols_is_null],
+        function(object) make_zero_col_DataFrame(length(object))
     )
     colnames_list <- lapply(mcols_list, colnames)
-    allCols <- unique(unlist(colnames_list, use.names=FALSE))
+    all_colnames <- unique(unlist(colnames_list, use.names=FALSE))
     fillCols <- function(df) {
         if (nrow(df))
-            df[setdiff(allCols, colnames(df))] <- DataFrame(NA)
+            df[setdiff(all_colnames, colnames(df))] <- DataFrame(NA)
         df
     }
     do.call(rbind, lapply(mcols_list, fillCols))
 }
 
-### We provide a default "concatenateObjects" method for Vector objects that
-### concatenates all the parallel slots. The method behaves like
-### an endomorphism with respect to its first argument 'x'.
-### Note that this method will work out-of-the-box and do the right thing
-### on most Vector subclasses as long as parallelSlotNames() reports the
-### names of all the parallel slots on objects of the subclass (some Vector
-### subclasses might require a "parallelSlotNames" method for this to happen).
-### For those Vector subclasses on which concatenateObjects() does not work
-### out-of-the-box nor do the right thing, it is strongly advised to override
-### the method for Vector objects rather than trying to override the "c"
-### method for Vector objects with a specialized method. The specialized
-### "concatenateObjects" method will typically delegate to the method below
-### via the use of callNextMethod(). See "concatenateObjects" methods for
+### We provide a default "bindROWS" method for Vector objects that uses
+### bindROWS() internally to concatenate the parallel slots along the ROWS.
+### The method behaves like an endomorphism with respect to its first
+### argument 'x'. Note that this method will work out-of-the-box and do the
+### right thing on most Vector subclasses as long as parallelSlotNames()
+### reports the names of all the parallel slots on objects of the subclass
+### (some Vector subclasses might require a "parallelSlotNames" method for
+### this to happen). For those Vector subclasses on which bindROWS() does not
+### work out-of-the-box nor do the right thing, it is strongly advised to
+### override the method for Vector objects rather than trying to override
+### the "c" method for Vector objects with a specialized method. The
+### specialized "bindROWS" method will typically delegate to the method
+### below via the use of callNextMethod(). See "bindROWS" methods for
 ### Hits and Rle objects for some examples.
 ### No Vector subclass should need to override the "c" method for
 ### Vector objects.
-.concatenate_Vector_objects <-
+concatenate_Vector_objects <-
     function(x, objects=list(), use.names=TRUE, ignore.mcols=FALSE, check=TRUE)
 {
     if (!isTRUEorFALSE(use.names))
@@ -618,7 +628,7 @@ rbind_mcols <- function(x, ...)
     if (!isTRUEorFALSE(check))
         stop("'check' must be TRUE or FALSE")
 
-    objects <- prepare_objects_to_concatenate(x, objects)
+    objects <- prepare_objects_to_bind(x, objects)
     all_objects <- c(list(x), objects)
 
     ## Concatenate all the parallel slots except "NAMES" and "elementMetadata".
@@ -630,7 +640,7 @@ rbind_mcols <- function(x, ...)
             if (is.null(x_slot))
                 return(NULL)
             slot_list <- lapply(objects, slot, slotname)
-            concatenateObjects(x_slot, slot_list)
+            bindROWS(x_slot, slot_list)
         }
     )
 
@@ -667,20 +677,19 @@ rbind_mcols <- function(x, ...)
     ans
 }
 
-setMethod("concatenateObjects", "Vector", .concatenate_Vector_objects)
+setMethod("bindROWS", "Vector", concatenate_Vector_objects)
 
-### Thin wrapper around concatenateObjects(). Behave like an endomorphism
-### i.e. return an object of the same class as 'x'. In particular 'c(x)'
-### should return 'x'.
+### Thin wrapper around bindROWS(). Behave like an endomorphism i.e. return
+### an object of the same class as 'x'. In particular 'c(x)' should return 'x'.
 ### No Vector subclass should need to override this method. See the
-### "concatenateObjects" method for Vector objects above for more information.
+### "bindROWS" method for Vector objects above for more information.
 setMethod("c", "Vector",
     function(x, ..., ignore.mcols=FALSE, recursive=FALSE)
     {
         if (!identical(recursive, FALSE))
             stop(wmsg("\"c\" method for Vector objects ",
                       "does not support the 'recursive' argument"))
-        concatenateObjects(x, list(...), ignore.mcols=ignore.mcols)
+        bindROWS(x, list(...), ignore.mcols=ignore.mcols)
     }
 )
 
