@@ -279,7 +279,7 @@ makeClassinfoRowForCompactPrinting <- function(x, col2class)
     right_brackets[!no_bracket] <- ">"
     ans <- paste0(left_brackets, col2class, right_brackets)
     names(ans) <- ans_names
-    x_mcols <- mcols(x)
+    x_mcols <- mcols(x, use.names=FALSE)
     x_nmc <- if (is.null(x_mcols)) 0L else ncol(x_mcols)
     if (x_nmc > 0L) {
         tmp <- sapply(x_mcols,
@@ -367,19 +367,25 @@ setMethod("classNameForDisplay", "AsIs",
 ### Exported!
 setGeneric("showAsCell", function(object) standardGeneric("showAsCell"))
 
-### Must work on any array-like object (i.e. on any object with 2 dimensions
-### or more) e.g. ordinary array or matrix, Matrix, data.frame, DataFrame,
-### data.table, etc...
+### Should work on any matrix-like object e.g. ordinary matrix, Matrix,
+### data.frame, DataFrame, data.table, etc...
+### Should also work on any array-like object with more than 2 dimensions
+### that supports "reshaping" via the dim() setter. Note that DelayedArray
+### objects don't support this reshaping in general.
 .showAsCell_array <- function(object)
 {
-    if (length(dim(object)) > 2L)
-        dim(object) <- c(nrow(object), prod(tail(dim(object), -1L)))
+    if (length(dim(object)) > 2L) {
+        ## Reshape 'object' as a 2D object.
+        dim1 <- dim(object)[-1L]
+        dim(object) <- c(nrow(object), prod(dim1))
+    }
     object_ncol <- ncol(object)
     if (object_ncol == 0L)
         return(rep.int("", nrow(object)))
-    object <- lapply(head(seq_len(object_ncol), 3L),
-                     function(i) showAsCell(object[ , i, drop=TRUE]))
-    ans <- do.call(paste, c(object, list(sep=":")))
+    first_cols <- lapply(seq_len(min(object_ncol, 3L)),
+        function(j) showAsCell(object[ , j, drop=TRUE])
+    )
+    ans <- do.call(paste, c(first_cols, list(sep=":")))
     if (object_ncol > 3L)
         ans <- paste0(ans, ":...")
     ans
@@ -387,8 +393,9 @@ setGeneric("showAsCell", function(object) standardGeneric("showAsCell"))
 
 .default_showAsCell <- function(object)
 {
-  ## Some objects like SplitDataFrameList are not array-like but have
-  ## a "dim" method that return a matrix!
+  ## Some objects like SplitDataFrameList have a "dim" method that
+  ## returns a non-MULL object (a matrix!) even though they don't have
+  ## an array-like semantic.
   if (length(dim(object)) >= 2L && !is.matrix(dim(object)))
     return(.showAsCell_array(object))
   object_NROW <- NROW(object)
