@@ -446,7 +446,7 @@ setGeneric("extractROWS", signature=c("x", "i"),
     function(x, i) standardGeneric("extractROWS")
 )
 
-setGeneric("replaceROWS", signature="x",
+setGeneric("replaceROWS", signature=c("x", "i"),
     function(x, i, value) standardGeneric("replaceROWS")
 )
 
@@ -457,7 +457,7 @@ default_extractROWS <- function(x, i)
   ## dynamically call [i,,,..,drop=FALSE] with as many "," as length(dim)-1
   ndim <- max(length(dim(x)), 1L)
   i <- normalizeSingleBracketSubscript(i, x, allow.NAs=TRUE)
-  args <- rep.int(alist(foo=), ndim)
+  args <- rep.int(list(quote(expr=)), ndim)
   args[[1]] <- i
   args <- c(list(x), args, list(drop=FALSE))
   do.call(`[`, args)
@@ -469,7 +469,7 @@ default_replaceROWS <- function(x, i, value)
     return(x)
   ndim <- max(length(dim(x)), 1L)
   i <- normalizeSingleBracketSubscript(i, x, allow.append=TRUE)
-  args <- rep.int(alist(foo=), ndim)
+  args <- rep.int(list(quote(expr=)), ndim)
   args[[1]] <- i
   args <- c(list(x), args, list(value=value))
   do.call(`[<-`, args)
@@ -547,7 +547,7 @@ subset_along_ROWS <- function(x, i, j, ..., drop=TRUE)
 
 setMethod("[", "LLint", subset_along_ROWS)
 
-setMethod("replaceROWS", "ANY", default_replaceROWS)
+setMethod("replaceROWS", c("ANY", "ANY"), default_replaceROWS)
 
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -720,7 +720,7 @@ setMethod("getListElement", "list",
     ## `[<-` propagates the metadata columns from 'value' to 'x' but here
     ## we don't want that.
     if (is(x, "Vector"))
-        x_mcols <- mcols(x)
+        x_mcols <- mcols(x, use.names=FALSE)
     x[i] <- value
     if (is(x, "Vector"))
         mcols(x) <- x_mcols
@@ -801,9 +801,9 @@ setMethod("tail", "LLint", tail.LLint)
 
 rep.int_along_ROWS <- function(x, times)
 {
-    x_len <- length(x)
     if (!(is.numeric(times) || is.LLint(times)))
         stop("'times' must be a numeric or LLint vector")
+    x_NROW <- NROW(x)
     times_len <- length(times)
     if (times_len == 1L) {
         if (times == 1L)
@@ -811,12 +811,12 @@ rep.int_along_ROWS <- function(x, times)
         if (times == 0L)
             return(extractROWS(x, integer(0)))
     }
-    if (times_len == x_len) {
-        i <- Rle(seq_len(x_len), times)
+    if (times_len == x_NROW) {
+        i <- Rle(seq_len(x_NROW), times)
     } else if (times_len == 1L) {
         if (is.LLint(times))
             times <- as.double(times)
-        i <- IRanges::IRanges(rep.int(1L, times), rep.int(x_len, times))
+        i <- IRanges::IRanges(rep.int(1L, times), rep.int(x_NROW, times))
     } else {
         stop("invalid 'times' value")
     }
