@@ -45,50 +45,24 @@ sapply_NROW <- function(x)
     return(vapply(x, NROW, integer(1)))
 }
 
-### TODO: Remove in BioC 3.8.
-listElementType <- function(x) {
-  .Defunct("lowestListElementClass")
-  cl <- lapply(x, class)
-  clnames <- unique(unlist(cl, use.names=FALSE))
-  if (length(clnames) == 1L) {
-    clnames
-  } else {
-    contains <- lapply(cl, function(x) getClass(x, TRUE)@contains)
-    clnames <- c(clnames,
-                 unlist(lapply(contains, names), use.names=FALSE))
-    cltab <- table(factor(clnames, unique(clnames)))
-    clnames <- names(cltab)[cltab == length(x)]
-    if (length(clnames) > 0L) {
-      clnames[1]
-    } else {
-      NULL
-    }
-  }
-}
-
-### A replacement for listElementType() that has a slightly different semantic.
-### - listElementType(): return the closest common ancestor class of all the
-###   list elements in 'x', or NULL.
-### - lowestListElementClass(): return the common ancestor class **among**
-###   the classes of the list elements in 'x', or "ANY". In other words, if
-###   all the classes in 'x' extend one of them, then lowestListElementClass()
-###   returns it. Otherwise, it returns "ANY".
-### As a consequence, unlike listElementType() which can return the name of a
-### virtual class (e.g. vector_OR_factor), lowestListElementClass() is
-### guaranteed to always return a **concrete** class or "ANY".
+### Return the common ancestor class **among** the classes of the list elements
+### in 'x', or "ANY". In other words, if all the classes in 'x' extend one of
+### them, then lowestListElementClass() returns it. Otherwise, it returns "ANY".
+### As a consequence, lowestListElementClass() is guaranteed to always return a
+### **concrete** class or "ANY".
 ###
 ### For example:
 ###
-###   classes in 'x'              listElementType      lowestListElementClass
-###   -------------------------   ------------------   ----------------------
-###   all the same                common class         common class
-###   integer,numeric             "numeric"            "numeric"
-###   integer,factor              "integer"            "integer"
-###   numeric,factor              "numeric"            "numeric"
-###   integer,numeric,character   "vector"             "ANY"
-###   character,factor            "vector_OR_factor"   "ANY"
-###   matrix, data.frame          "vector"             "ANY"
-###   character,list              "vector"             "ANY"
+###   classes in 'x'              lowestListElementClass
+###   -------------------------   ----------------------
+###   all the same                common class
+###   integer,numeric             "numeric"
+###   integer,factor              "integer"
+###   numeric,factor              "numeric"
+###   integer,numeric,character   "ANY"
+###   character,factor            "ANY"
+###   matrix, data.frame          "ANY"
+###   character,list              "ANY"
 ###
 lowestListElementClass <- function(x)
 {
@@ -116,13 +90,13 @@ lowestListElementClass <- function(x)
 
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### Concatenation
+### Concatenation along the ROWS
 ###
 
 ### Exported!
-### Works on atomic vectors, factors, lists, matrices, and data frames.
-### Arguments 'ignore.mcols' and 'check' are ignored.
-.concatenate_vectors <-
+### Works on atomic vectors, factors, lists, 1D arrays, matrices, and
+### data frames. Arguments 'ignore.mcols' and 'check' are ignored.
+.default_bindROWS <-
     function(x, objects=list(), use.names=TRUE, ignore.mcols=FALSE, check=TRUE)
 {
     if (!is.list(objects))
@@ -130,7 +104,7 @@ lowestListElementClass <- function(x)
     if (!isTRUEorFALSE(use.names))
         stop("'use.names' must be TRUE or FALSE")
 
-    ## We do not call prepare_objects_to_concatenate() because we do not want
+    ## We do not call prepare_objects_to_bind() because we do not want
     ## to force all the objects in 'objects' to be of the type of 'x'. This
     ## way we are consistent with what c() and unlist() do when combining
     ## atomic vectors of mixed types.
@@ -139,25 +113,68 @@ lowestListElementClass <- function(x)
             if (is(object, "Rle")) decodeRle(object) else object)
     all_objects <- c(list(x), objects)
 
-    if (length(dim(x)) == 2L) {
+    x_ndim <- length(dim(x))
+    if (x_ndim == 0L) {
+        ## Use unlist() if 'x' is an atomic vector, a factor, or a list.
+        ## Otherwise use c().
+        if (is.vector(x) || is.factor(x)) {
+            ans <- unlist(all_objects, recursive=FALSE)
+        } else {
+            ans <- do.call(c, all_objects)
+        }
+        if (!use.names)
+            names(ans) <- NULL
+    } else if (x_ndim == 1L) {
+        ## 'x' is a 1D array.
+        ## base::rbind() is broken on 1D arrays so we need to handle this
+        ## specially.
+        ## Note that all objects in 'objects' are also treated as if they
+        ## were 1D arrays (even if they have >= 2 dimensions). This is
+        ## probably too laxist!
+        ans <- unlist(all_objects, recursive=FALSE)
+        if (use.names)
+            ans_rownames <- names(ans)
+        dim(ans) <- length(ans)  # this drops the names
+        if (use.names)
+            rownames(ans) <- ans_rownames
+    } else if (x_ndim == 2L) {
+        ## 'x' is a matrix or data frame.
         ans <- do.call(rbind, all_objects)
         if (!use.names)
             rownames(ans) <- NULL
     } else {
-        ans <- unlist(all_objects, recursive=FALSE)
-        if (!use.names)
-            names(ans) <- NULL
+        ## 'x' is an array with more than 2 dimensions.
+        ## Binding multi-dimensional arrays along the rows is exactly what
+        ## the DelayedArray::arbind() generic does so we should probably move
+        ## this generic to S4Vectors (or to BiocGenerics?).
+        stop(wmsg("bindROWS() does not support arrays ",
+                  "with more than 2 dimensions yet"))
     }
     ans
 }
 
-setMethod("concatenateObjects", "vector", .concatenate_vectors)
-setMethod("concatenateObjects", "matrix", .concatenate_vectors)
+### Even though is(x, "vector") and is.vector(x) are FALSE when 'x'
+### is a data frame, calling bindROWS() on 'x' will actually dispatch
+### on the bindROWS,vector method (this can be checked with
+### selectMethod("bindROWS", "data.frame")) so we don't need to
+### define a bindROWS,data.frame method.
+#setMethod("bindROWS", "vector", .default_bindROWS)
+
+### Even though calling bindROWS() on an array would dispatch on the
+### bindROWS,vector method (is(x, "vector") is TRUE) we still need to
+### define the bindROWS,array method. Otherwise, the dispatch mechanism
+### seems to remove the dim attribute from 'x' **before** passing it to
+### the bindROWS,vector method.
+### See https://stat.ethz.ch/pipermail/r-devel/2018-May/076205.html for
+### the bug report.
+#setMethod("bindROWS", "array", .default_bindROWS)
+
+setMethod("bindROWS", "ANY", .default_bindROWS)
 
 ### Assumes that 'x' is a list of length >= 1 with no names, and that the
 ### list elements in 'x' have the same type. This is NOT checked!
-### TODO: quick_unlist() is superseded by concatenateObjects(). Search code
-### for use of quick_unlist() and replace with use of concatenateObjects().
+### TODO: quick_unlist() is superseded by bindROWS(). Search code
+### for use of quick_unlist() and replace with use of bindROWS().
 ### Then remove quick_unlist() definition below.
 quick_unlist <- function(x)
 {
