@@ -246,20 +246,9 @@ test_Rle_table <- function()
 }
 
 test_Rle_Integer_overflow <- function() {
-
-    x0 <- Rle(as.integer(c(1,(2^31)-1,1)))
-    checkIdentical(NA_integer_, suppressWarnings(sum(x0)))
-
-    testWarning <- NULL
-    suppressWarnings(withCallingHandlers({sum(x0)
-    }, warning=function(warn) {
-        msg <- conditionMessage(warn)
-        exp <- gettext("integer overflow - use runValue(.) <- as.numeric(runValue(.))",
-                       domain="R")
-        if (msg == exp)
-            testWarning <<- TRUE 
-    }))
-    checkTrue(testWarning)
+    v <- as.integer(c(1,(2^31)-1,1))
+    x0 <- Rle(v)
+    checkIdentical(sum(v), sum(x0))
 
     x <- Rle(c(1,(2^31)-1,1))
     checkIdentical(mean(x0), mean(x))
