@@ -29,6 +29,11 @@ int _safe_int_mult(
 	int y
 );
 
+int _as_int(
+	const char *val,
+	int val_len
+);
+
 long long int _safe_llint_add(
 	long long int x,
 	long long int y
@@ -534,12 +539,12 @@ SEXP new_CHARACTER_from_LLint(SEXP x);
 
 /* subsetting_utils.c */
 
-int _copy_vector_block(
+long long int _copy_vector_block(
 	SEXP dest,
-	int dest_offset,
+	long long int dest_offset,
 	SEXP src,
-	int src_offset,
-	int block_width
+	long long int src_offset,
+	long long int block_nelt
 );
 
 int _copy_vector_positions(
@@ -901,7 +906,9 @@ SEXP select_hits(
 	SEXP from,
 	SEXP to,
 	SEXP nLnode,
-	SEXP select
+	SEXP nRnode,
+	SEXP select,
+	SEXP nodup
 );
 
 SEXP make_all_group_inner_hits(
